#include "Bound.h"


using namespace std;
using namespace glm;

Ball::Ball(){
}

Ball::Ball(float radius, float bounciness, glm::vec3 location, glm::vec3 initVel) {
   loc = location;
	velocity = initVel;
	rad = radius;
	bounce = bounciness;
}

//Change the velocity of the ball so it bounces appropriatly
vec3 Ball::Bounce(vec3 wall) {
	velocity = -2*(dot(wall, velocity))*wall + velocity;
	velocity *= bounce;
	return velocity;
}

//Move the ball to a given location
vec3 Ball::MoveTo(vec3 toMove) {
	loc = toMove;
}

//return the location of the ball after it would move. Does not move the ball though
vec3 Ball::CheckMove() {
	return loc + velocity;
}

//change the velocity of the ball
bool Ball::Accelerate(vec3 change) {
	velocity += change;
}

//check to see if the ball collided with another ball
bool Ball::checkCollisionBall(glm::vec3 ball, float ballRad) {
   if (sqrt((loc.x - ball.x)*(loc.x - ball.x) + (loc.y - ball.y)*(loc.y - ball.y) + (loc.z - ball.z)*(loc.z - ball.z)) < rad + ballRad)
      return true;

   return false;
}

/*end of Ball, start of static bounding box*/

   StaticBB::StaticBB() {
   }
   
	StaticBB::StaticBB(glm::vec3 coord, float length, float height, float width) {
		float longest = length;
		
		if (longest < height)
			longest = height;
		if (longest < width)
			longest = width;
			
		loc = coord;
		len = length;
		wid = width;
		ht = height;
		hLen = length/2.0;
		hWid = width/2.0;
		hHt = height/2.0;
		radius = longest*.75;
	}
	
	//this can be made more efficient (can remove the dot product calculation). Checks to see if a ball collided with the bounding box
	bool StaticBB::checkCollisionBall(glm::vec3 ball, float ballRad) {
		vec3 MaxV, MinV;
		
		//check if the ball is close enough in range to do the calculations
		if (sqrt((loc.x - ball.x)*(loc.x - ball.x) + (loc.y - ball.y)*(loc.y - ball.y) + (loc.z - ball.z)*(loc.z - ball.z)) <= radius + ballRad) {
			MaxV = (ball - vec3(loc.x + hLen, loc.y + hHt, loc.z + hWid));
			MinV = (ball - vec3(loc.x - hLen, loc.y - hHt, loc.z - hWid));
			//check if the ball is not above the box, to the right of the box, and behind the box
			if (dot(MaxV, vec3(1, 0, 0)) - ballRad < 0 && dot(MaxV, vec3(0, 1, 0)) - ballRad < 0 && dot(MaxV, vec3(0, 0, 1)) - ballRad < 0) {
				//check if the ball is not below the box, to the left of the box, and in front of the box
				if (dot(MinV, vec3(-1, 0, 0)) - ballRad < 0 && dot(MinV, vec3(0, -1, 0)) - ballRad < 0 && dot(MinV, vec3(0, 0, -1)) - ballRad < 0) {
					return true;
				}
			}
		}
		
		return false;
	}
	
	//checks to see which wall was probably passed through. This does have some issues as it assumes the wall closest to the ball was passed through. I had segfaults when I checked to see which wall was not passed through with the pre collided position. This is because the ball would be moved into the the wall after colliding with another ball after this calculation occured. 
	vec3 StaticBB::WhichWall(glm::vec3 ball, float ballRad) {
	   vec3 maxPre = ball - vec3(loc.x + hLen, loc.y + hHt, loc.z + hWid);
	   vec3 minPre = ball - vec3(loc.x - hLen, loc.y - hHt, loc.z - hWid);
	   vec3 ret = vec3(0, 1, 0);
	   float distDown, distRight, distLeft, distFront, distBack, closest;
	   
	   //check if above it
	   closest = fabs(dot(maxPre, vec3(0, 1, 0)) - ballRad);
	   //check if below it
	   distDown = fabs(dot(minPre, vec3(0, -1, 0)) - ballRad);
	   //check if in front of it
	   distFront = fabs(dot(minPre, vec3(0, 0, -1)) - ballRad);
	   //check if behind it
	   distBack = fabs(dot(maxPre, vec3(0, 0, 1)) - ballRad);
	   //check if to the right of it
	   distRight = fabs(dot(maxPre, vec3(1, 0, 0)) - ballRad);
	   //check if to the left of it
	   distLeft = fabs(dot(minPre, vec3(-1, 0, 0)) - ballRad);
	   
   	if (distDown <= closest) {
   		ret = vec3(0, -1, 0);
   		closest = distDown;
   	}
   	if (distLeft <= closest) {
   		ret = vec3(-1, 0, 0);
   		closest = distLeft;   		
   	}
   	if (distRight <= closest) {
   		ret = vec3(1, 0, 0);
   		closest = distRight;	
   	}
   	if (distFront <= closest) {
   		ret = vec3(0, 0, -1);
   		closest = distFront;		
   	}
   	if (distBack <= closest) {
   		ret = vec3(0, 0, 1);
   		closest = distBack;	
   	}
	   	
	   return closest*normalize(ret);
	}
	
	//checks to see if two bounding boxes collided. If so, it returns a vector of which sides were most likely collided with
	vec3 StaticBB::checkCollisionBB(vec3 preMove, StaticBB otherBox) {
		float b1MaxX, b1MaxY, b1MaxZ, b1MinX, b1MinY, b1MinZ;
		float b2MaxX, b2MaxY, b2MaxZ, b2MinX, b2MinY, b2MinZ;
		float preMaxX, preMaxY, preMaxZ, preMinX, preMinY, preMinZ;
		vec3 ret = vec3(0, 0, 0);
		
		//check to see if the boxes are within range to even collide
		if (sqrt((loc.x - otherBox.loc.x)*(loc.x - otherBox.loc.x) + (loc.y - otherBox.loc.y)*(loc.y - otherBox.loc.y) + (loc.z - otherBox.loc.z)*(loc.z - otherBox.loc.z)) <= radius + otherBox.radius) {
			//calculate the locations of box end points
			b1MaxX = loc.x + hLen;
			b1MaxY = loc.y + hHt;
			b1MaxZ = loc.z + hWid; 
			b1MinX = loc.x - hLen;  
			b1MinY = loc.y - hHt;
			b1MinZ = loc.z - hWid;
			b2MaxX = otherBox.loc.x + otherBox.hLen;
			b2MaxY = otherBox.loc.y + otherBox.hHt; 
			b2MaxZ = otherBox.loc.z + otherBox.hWid; 
			b2MinX = otherBox.loc.x - otherBox.hLen;
			b2MinY = otherBox.loc.y - otherBox.hHt;
			b2MinZ = otherBox.loc.z - otherBox.hWid;
			if (b1MaxX > b2MinX && b1MinX < b2MaxX && b1MaxY > b2MinY && b1MinY < b2MaxY && b1MaxZ > b2MinZ && b1MinZ < b2MaxZ) {
				ret = vec3(1, 1, 1);
				preMaxX = preMove.x + otherBox.hLen;
				preMaxY = preMove.y + otherBox.hHt;
				preMaxZ = preMove.z + otherBox.hWid; 
				preMinX = preMove.x - otherBox.hLen;  
				preMinY = preMove.y - otherBox.hHt;
				preMinZ = preMove.z - otherBox.hWid;
				if (!(b1MaxX > preMinX && b1MinX < preMaxX))
					ret.x = 0;
				if (!(b1MaxY > preMinY && b1MinY < preMaxY))
					ret.y = 0;
				if (!(b1MaxZ > preMinZ && b1MinZ < preMaxZ))
					ret.z = 0;
			}
		}
		
		return ret;
	}
	
	//moves the bounding box to the given location
	vec3 StaticBB::updateLoc(glm::vec3 update)  {
		loc = update;
	}

	/*changing from static bounding box to moving/rotating bounding boxes*/

	SpecialBB::SpecialBB() {
		groupAPt = vec4(.5f, .5f, .5f, 1);
		groupBPt = vec4(-.5f, -.5f, -.5f, 1);
		loc = vec4(0, 0, 0, 1);
		groupAX = vec4(1, 0, 0, 0);
		groupAY = vec4(0, 1, 0, 0);
		groupAZ = vec4(0, 0, 1, 0);
		groupBX = vec4(-1, 0, 0, 0);
		groupBY = vec4(0, -1, 0, 0);
		groupBZ = vec4(0, 0, -1, 0);
		matrix = mat4(1.0f);	
		radius = 1;
	}

   SpecialBB::SpecialBB(glm::mat4 initMatrix, float rad) {
		groupAPt = initMatrix*vec4(.5f, .5f, .5f, 1);
		groupBPt = initMatrix*vec4(-.5f, -.5f, -.5f, 1);
		loc = matrix*vec4(0, 0, 0, 1);
		groupAX = initMatrix*vec4(1, 0, 0, 0);
		groupAY = initMatrix*vec4(0, 1, 0, 0);
		groupAZ = initMatrix*vec4(0, 0, 1, 0);
		groupBX = initMatrix*vec4(-1, 0, 0, 0);
		groupBY = initMatrix*vec4(0, -1, 0, 0);
		groupBZ = initMatrix*vec4(0, 0, -1, 0);
		matrix = initMatrix;
		radius = rad;
   }
   
   /*Checks to see if the box collided with a ball*/
   bool SpecialBB::checkCollisionBall(glm::vec3 ball, float ballRad) {
   	vec3 groupAVec, groupBVec;
		
		//check if the ball is in range to collide with it
		if (sqrt((loc.x - ball.x)*(loc.x - ball.x) + (loc.y - ball.y)*(loc.y - ball.y) + (loc.z - ball.z)*(loc.z - ball.z)) < ballRad + radius) {
			groupAVec = (ball - vec3(groupAPt));
			groupBVec = (ball - vec3(groupBPt));
			//check if the ball is within the faces of group A
			if (dot(groupAVec, vec3(groupAX)) - ballRad <= 0 && dot(groupAVec, vec3(groupAY)) - ballRad <= 0 && dot(groupAVec, vec3(groupAZ)) - ballRad <= 0) {
				//check if the ball is within the faces of group B
				if (dot(groupBVec, vec3(groupBX)) - ballRad <= 0 && dot(groupBVec, vec3(groupBY)) - ballRad <= 0 && dot(groupBVec, vec3(groupBZ)) - ballRad <= 0) {
					return true;
				}
			}		
		}
		return false;
   }

	/*Returns an approximation of which wall the ball collided with. Same method as non-rotating boundig box*/
   glm::vec3 SpecialBB::WhichWall(glm::vec3 ball, float ballRad) {
		float distAY = fabs(dot(ball - vec3(groupAPt), vec3(groupAY)) - ballRad);
		float distAZ = fabs(dot(ball - vec3(groupAPt), vec3(groupAZ)) - ballRad);
		float distBX = fabs(dot(ball - vec3(groupBPt), vec3(groupBX)) - ballRad);
		float distBY = fabs(dot(ball - vec3(groupBPt), vec3(groupBY)) - ballRad);
		float distBZ = fabs(dot(ball - vec3(groupBPt), vec3(groupBZ)) - ballRad); 
		/*distAX stored in closest*/
		float closest =  fabs(dot(ball - vec3(groupAPt), vec3(groupAX)) - ballRad);
		vec4 ret = groupAX;
		
		if (distAY <= closest) {
   		ret = groupAY;
   		closest = distAY;
   	}
   	if (distAZ <= closest) {
   		ret = groupAZ;
   		closest = distAZ;   		
   	}
   	if (distBX <= closest) {
   		ret = groupBX;
   		closest = distBX;	
   	}
   	if (distBY <= closest) {
   		ret = groupBY;
   		closest = distBY;		
   	}
   	if (distBZ <= closest) {
   		ret = groupBZ;
   		closest = distBZ;	
   	}
		
		return closest*normalize(vec3(ret));
   }

	/*changes everything in the bounding box to match the new matrix for it*/
	void SpecialBB::ChangeMatrix(glm::mat4 newMat) {
		matrix = newMat;	
		groupAPt = matrix*vec4(.5f, .5f, .5f, 1);
		groupBPt = matrix*vec4(-.5f, -.5f, -.5f, 1);
		loc = matrix*vec4(0, 0, 0, 1);
		groupAX = matrix*vec4(1, 0, 0, 0);
		groupAY = matrix*vec4(0, 1, 0, 0);
		groupAZ = matrix*vec4(0, 0, 1, 0);
		groupBX = matrix*vec4(-1, 0, 0, 0);
		groupBY = matrix*vec4(0, -1, 0, 0);
		groupBZ = matrix*vec4(0, 0, -1, 0);
	}
	
	/*change the radius of the bounding box. so that it can do all the calculations potential collisions only on objects in range to collide with it*/
	void SpecialBB::ScaleRad(float scale) {
		radius*=scale;
	}
	
	
