/*
 *  CPE 471 lab 4 - draws a box without sides
 *  glut/OpenGL application 
 *  uses glm for matrices
 *  Has support for simple transforms - check the keyboard function
 *
 *  Created by zwood on 1/6/12 
 *  Copyright 2010 Cal Poly. All rights reserved.
 *
 *****************************************************************************/
#ifdef __APPLE__
#include "GLUT/glut.h"
#include <OPENGL/gl.h>
#endif
#ifdef __unix__
#include <GL/glut.h>
#endif

#include "CMeshLoaderSimple.h"
#include <stdlib.h>
#include <stdio.h>
#include "GLSL_helper.h"
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr

#include "InputFunctions.h"
#include "HelperFunctions.h"
#include "GeometryCreator.h"
#include "MStackHelp.h"
#include "Bound.h"

#define TIMER_MS 23
#define LEGS 1
#define BALL 8
#define GROUNDTILE 0
#define WALL 1
#define RED 2
#define SKIN 9

using namespace std;

//declare a matrix stack
RenderingHelper ModelTrans;

//flag and ID to toggle on and off the shader
int ShadeProg;

//Handles to the shader data
GLint h_aNormal, h_aPosition;
GLint h_uModelMatrix, h_uViewMatrix, h_uProjMatrix;
GLint h_uMatAmb, h_uMatDif, h_uMatSpec, h_uMatShine;

float g_width, g_height;

//adding pitch and yaw, as well as eye moving stuff
float pitch = 0.0, yaw = -3.14/2;
float startX = 0.0, startY = 0.0;

glm::vec3 vecW, vecU, vecV;
glm::vec3 eye(0, 0, 0);
glm::vec3 gaze(cos(pitch)*cos(yaw), sin(pitch), cos(pitch)*cos(3.14/2 - yaw));

unsigned int const MeshCount = 14;
Mesh * Meshes[MeshCount];

/*Ball Things*/
Ball bouncies[8];
int ballCount = 0;
int ballCols[8] = {0, 0, 0, 0, 0, 0, 0, 0};
/*Wall boxes (includes player)*/
StaticBB playerBox;
StaticBB walls[10];
int wallCount = 7;
SpecialBB SpecialWall[14];
int specWallCount = 14;
float rotAng = 0;

float lightPosArray[3] = { 0, 20, 0};
float lightColArray[3] = {20, 20, 20};

float **groundArray;
int groundLen = 16;

/*velocities and gravity*/
float velX = 0, velY = 0, velZ = 0, gravity = -.02, speed = .1;

/*array of flags (can be replaced with a single char you can do bit ops on, but I will worry about that later*/
bool moveFlags[5] = {false , false, false, false, false};
bool canJump = true;

/* model transforms - change these to create a shape with boxes*/
void SetModel() {
  safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(ModelTrans.modelViewMatrix));
}
/* helper function to set up material for shading */
void SetMaterial(int i) {

	glUseProgram(ShadeProg);
	switch (i) {
		
		//starting ball color
		case 0:
			glUniform3f(h_uMatAmb, 0.2, 0.2, 0.2);
			glUniform3f(h_uMatDif, 0.0, 0.08, 0.5);
			glUniform3f(h_uMatSpec, 0.4, 0.4, 0.4);
			safe_glUniform1f(h_uMatShine, 200.0);
			break;
		
		//ground color
		case 1:
			glUniform3f(h_uMatAmb, 0.1, 0.2, 0.05);
			glUniform3f(h_uMatDif, 0.5, 0.7, 0.1);
			glUniform3f(h_uMatSpec, 0.3, 0.3, 0.3);
			safe_glUniform1f(h_uMatShine, .5);
			break;
		
		//unused color for testing
		case RED:
        glUniform3f(h_uMatAmb, 0.2, 0.2, 0.2);
        glUniform3f(h_uMatDif, 0.5, 0.08, 0.0);
        glUniform3f(h_uMatSpec, 0.4, 0.4, 0.4);
        safe_glUniform1f(h_uMatShine, 200.0);
        break;
		
		//color balls turn into when colliding with balls
		case 3:
			glUniform3f(h_uMatAmb, 0.1, 0.2, 0.05);
			glUniform3f(h_uMatDif, 0.7, 0.4, 0.1);
			glUniform3f(h_uMatSpec, 0.4, 0.3, 0.2);
			safe_glUniform1f(h_uMatShine, 800.0);
			break;
		
		//color balls turn into when colliding with walls
		case 4:
			glUniform3f(h_uMatAmb, 0.1, 0.1, 0.1);
			glUniform3f(h_uMatDif, 0.8, 0.4, 0.8);
			glUniform3f(h_uMatSpec, 0.2, 0.4, 0.2);
			safe_glUniform1f(h_uMatShine, 100.0);
			break;
      
      //brick wall
		case 5:
			glUniform3f(h_uMatAmb, 0.23, 0.1, 0.1);
			glUniform3f(h_uMatDif, 0.8, 0.4, 0.4);
			glUniform3f(h_uMatSpec, 0.1, 0.1, 0.1);
			safe_glUniform1f(h_uMatShine, 2.0);
			break;
		
		//sky
		case 6:
        glUniform3f(h_uMatAmb, 0.3, 0.6, 1.0);
        glUniform3f(h_uMatDif, 0.0, 0.0, 0.0);
        glUniform3f(h_uMatSpec, 0.3, 0.3, 0.3);
        safe_glUniform1f(h_uMatShine, 40.0);
        break;
      
      //shiny black balls turn into when colliding with player
      case 7:
        glUniform3f(h_uMatAmb, 0.0, 0.0, 0.0);
        glUniform3f(h_uMatDif, 0.01, 0.01, 0.01);
        glUniform3f(h_uMatSpec, 0.3, 0.3, 0.3);
        safe_glUniform1f(h_uMatShine, 40.0);
        break;
        
      //wood
      case 8:
        glUniform3f(h_uMatAmb, 0.2, 0.2, 0.0);
        glUniform3f(h_uMatDif, 0.4, 0.4, 0.0);
        glUniform3f(h_uMatSpec, 0.3, 0.3, 0.3);
        safe_glUniform1f(h_uMatShine, 5.0);
        break;
      
      case SKIN:
        glUniform3f(h_uMatAmb, 0.2, .168, 0.177);
        glUniform3f(h_uMatDif, 1.0, 0.68, 0.77);
        glUniform3f(h_uMatSpec, 0.1, 0.1, 0.1);
        safe_glUniform1f(h_uMatShine, 50.0);
        break;
	}
}

/* Initialize the geometry */
void InitGeom() {
	ModelTrans.useModelViewMatrix();
   ModelTrans.loadIdentity();
   
   Meshes[GROUNDTILE] =  GeometryCreator::CreateCube(glm::vec3(1.f, .5f, 1.f));
   Meshes[WALL] =  GeometryCreator::CreateCube(glm::vec3(1.f, 1.f, 1.f));
   Meshes[2] =  GeometryCreator::CreateCylinder(1.7f, .01f, 1.5f, 8, 8);
   Meshes[3] =  GeometryCreator::CreateCylinder(1.f, 1.f, 1.2f, 8, 8);
   Meshes[4] =  GeometryCreator::CreateSphere(glm::vec3(.5f));
   Meshes[5] =  GeometryCreator::CreateDisc(0.5f, 1.5f, 1.75f, 32, 3);
   Meshes[6] =  GeometryCreator::CreateCylinder(.1f, 26.f, 10.f, 8, 8);
   Meshes[7] =  GeometryCreator::CreateSphere(glm::vec3(1.5f));
   Meshes[BALL] =  GeometryCreator::CreateSphere(glm::vec3(.2f));
   Meshes[9] =  GeometryCreator::CreateSphere(glm::vec3(2.0f), 4, 2);
   Meshes[10] = GeometryCreator::CreateSphere(glm::vec3(2.0f), 6, 3);
   Meshes[11] = GeometryCreator::CreateTorus(1.f, 2.f, 12, 24);
   Meshes[12] = GeometryCreator::CreateTorus(0.5f, 2.5f, 8, 12);
   Meshes[13] = GeometryCreator::CreateTorus(2.f, 2.5f, 48, 64);
}

/*takes in a number, then draws the mesh of that number*/
void drawMesh(int mesh) {
	SetModel();
	safe_glEnableVertexAttribArray(h_aPosition);
	glBindBuffer(GL_ARRAY_BUFFER, Meshes[mesh]->PositionHandle);
	safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);

	safe_glEnableVertexAttribArray(h_aNormal);
	glBindBuffer(GL_ARRAY_BUFFER, Meshes[mesh]->NormalHandle);
	safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, Meshes[mesh]->IndexHandle);

	glDrawElements(GL_TRIANGLES, Meshes[mesh]->IndexBufferLength, GL_UNSIGNED_SHORT, 0);

	safe_glDisableVertexAttribArray(h_aPosition);
}

/* Main display function */
void Draw (void)
{	
	int i, j;
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);				
	//Start our shader	
 	glUseProgram(ShadeProg);

   /* set up the projection and camera - do not change */
   SetProjectionMatrix();
   SetView();
   ModelTrans.loadIdentity();
       
	/* TO DO set up the light's position and color */
   glUniform3fv(safe_glGetUniformLocation(ShadeProg, "uLightPosArray"), 3, lightPosArray);
   glUniform3fv(safe_glGetUniformLocation(ShadeProg, "uLightColArray"), 3, lightColArray);
	
	/*draw ground*/
	SetMaterial(1);
	for(i = 0; i < groundLen; ++i) {
		for(j = 0; j < groundLen; ++j) {
			ModelTrans.pushMatrix();
			ModelTrans.translate(vec3(i-groundLen/2, groundArray[i][j]-.25, j-groundLen/2));
			drawMesh(GROUNDTILE);
			ModelTrans.popMatrix();
		}
	}
	
	/*draw balls*/
	for(i = 0; i < ballCount; ++i) {
		SetMaterial(ballCols[i]);
	   ModelTrans.pushMatrix();
		ModelTrans.translate(bouncies[i].loc);
		drawMesh(BALL);
		ModelTrans.popMatrix();
	}
	
	/*draw walls*/
	SetMaterial(5);
	for (i = 0; i < wallCount; ++i) {
		ModelTrans.pushMatrix();
		ModelTrans.translate(walls[i].loc);
		ModelTrans.scale(walls[i].len, walls[i].ht, walls[i].wid);
		drawMesh(WALL);
		ModelTrans.popMatrix();
	}
	
	//the box floating in the sky
	ModelTrans.pushMatrix();
		ModelTrans.translate(vec3(1, 3, 0));
		ModelTrans.rotate(rotAng, vec3(1, 1, 1));
		ModelTrans.scale(1, 1, 1);
		SpecialWall[0].ChangeMatrix(ModelTrans.modelViewMatrix);
		drawMesh(WALL);
	ModelTrans.popMatrix();
	
	//the sky
	SetMaterial(6);
	ModelTrans.pushMatrix();
		ModelTrans.translate(vec3(0, 15, 0));
		ModelTrans.scale(groundLen + 1, .5, groundLen + 1);
		drawMesh(WALL);
	ModelTrans.popMatrix();
	
	//windmill
	SetMaterial(8);
	ModelTrans.pushMatrix();
		ModelTrans.translate(vec3(0, 3, 6));
		ModelTrans.rotate(rotAng, vec3(0, 0, 1));
		ModelTrans.scale(.75, .75, .75);
		
		ModelTrans.pushMatrix();
			ModelTrans.translate(vec3(.25, 0, 0));
			ModelTrans.scale(2, 1, 1);
			ModelTrans.translate(vec3(1, 0, 0));
			SpecialWall[2].ChangeMatrix(ModelTrans.modelViewMatrix);
			drawMesh(WALL);
		ModelTrans.popMatrix();
		
		ModelTrans.pushMatrix();
			ModelTrans.translate(vec3(-.25, 0, 0));
			ModelTrans.scale(2, 1, 1);
			ModelTrans.translate(vec3(-1, 0, 0));
			SpecialWall[3].ChangeMatrix(ModelTrans.modelViewMatrix);
			drawMesh(WALL);
		ModelTrans.popMatrix();
		
		ModelTrans.pushMatrix();
			ModelTrans.translate(vec3(0, .25, 0));
			ModelTrans.scale(1, 2, 1);
			ModelTrans.translate(vec3(0, 1, 0));
			SpecialWall[4].ChangeMatrix(ModelTrans.modelViewMatrix);
			drawMesh(WALL);
		ModelTrans.popMatrix();
		
		ModelTrans.pushMatrix();
			ModelTrans.translate(vec3(0, -.25, 0));
			ModelTrans.scale(1, 2, 1);
			ModelTrans.translate(vec3(0, -1, 0));
			SpecialWall[5].ChangeMatrix(ModelTrans.modelViewMatrix);
			drawMesh(WALL);
		ModelTrans.popMatrix();
		
		SpecialWall[1].ChangeMatrix(ModelTrans.modelViewMatrix);
		drawMesh(WALL);
	ModelTrans.popMatrix();
	
	ModelTrans.pushMatrix();
	   ModelTrans.translate(vec3(0, 1.4, -2));
	   ModelTrans.scale(.3, .3, .3);
	   /* head start*/
	   ModelTrans.pushMatrix();
	   	ModelTrans.rotate(0, vec3(1, 0, 0));
         ModelTrans.scale(.5, 1, .5);
         ModelTrans.translate(vec3(0.0, 2.0, 0.0));
         SpecialWall[6].ChangeMatrix(ModelTrans.modelViewMatrix);
         SetModel();
         SetMaterial(SKIN);
         drawMesh(0);
      ModelTrans.popMatrix();
	   /* head end*/
	   /* upper arm start */
	   ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(-.75, 1.0, 0.0));
            ModelTrans.rotate( 0, vec3(1, 0, 0));
            ModelTrans.rotate(-90, vec3(0.0, 1.0, 0.0));
            ModelTrans.translate(vec3(.25, 0.0, 0.0));
            /* lower arm start */
            ModelTrans.pushMatrix();
            	
               ModelTrans.translate(vec3(0.75, 0.0, 0.0));
               ModelTrans.rotate(90, vec3(1.0, 0.0, 0.0));
               ModelTrans.scale(1, .5, .5);
               ModelTrans.translate(vec3(.25, 0.0, 0.0));
               SpecialWall[7].ChangeMatrix(ModelTrans.modelViewMatrix);
               SetModel();
               SetMaterial(0);
               drawMesh(0);
            ModelTrans.popMatrix();
            /* lower arm end */
         ModelTrans.scale(1, 1, .5);
         SpecialWall[8].ChangeMatrix(ModelTrans.modelViewMatrix);
         SetModel();
         SetMaterial(RED);
         drawMesh(0);
      ModelTrans.popMatrix();
      /* upper arm end */
      
      /* upper arm start */
	   ModelTrans.pushMatrix();
	   		ModelTrans.scale(-1, 1, 1);
            ModelTrans.translate(vec3(-.75, 1.0, 0.0));
            ModelTrans.rotate( 0, vec3(1, 0, 0));
            ModelTrans.rotate(-90, vec3(0.0, 1.0, 0.0));
            ModelTrans.translate(vec3(.25, 0.0, 0.0));
            /* lower arm start */
            ModelTrans.pushMatrix();
            	
               ModelTrans.translate(vec3(0.75, 0.0, 0.0));
               ModelTrans.rotate(90, vec3(1.0, 0.0, 0.0));
               ModelTrans.scale(1, .5, .5);
               ModelTrans.translate(vec3(.25, 0.0, 0.0));
               SpecialWall[9].ChangeMatrix(ModelTrans.modelViewMatrix);
               SetModel();
               SetMaterial(0);
               drawMesh(0);
            ModelTrans.popMatrix();
            /* lower arm end */
         ModelTrans.scale(1, 1, .5);
         SpecialWall[10].ChangeMatrix(ModelTrans.modelViewMatrix);
         SetModel();
         SetMaterial(RED);
         drawMesh(0);
      ModelTrans.popMatrix();
      /* upper arm end */
      /*Left leg*/
      ModelTrans.pushMatrix();
      	ModelTrans.translate(vec3(.14, -1.3, 0.0));
      	//rotation here
      	ModelTrans.scale(.4, 3, 1);
      	ModelTrans.translate(vec3(.25, 0.0, 0.0));
      	SpecialWall[11].ChangeMatrix(ModelTrans.modelViewMatrix);
      	SetModel();
      	SetMaterial(0);
      	drawMesh(0);
      ModelTrans.popMatrix();
      
      /*right leg*/
      ModelTrans.pushMatrix();
      	ModelTrans.translate(vec3(-.36, -1.3, 0.0));
      	//rotation here
      	ModelTrans.scale(.4, 3, 1);
      	ModelTrans.translate(vec3(.25, 0.0, 0.0));
      	SpecialWall[12].ChangeMatrix(ModelTrans.modelViewMatrix);
      	SetModel();
      	SetMaterial(0);
      	drawMesh(0);
      ModelTrans.popMatrix();
      
      ModelTrans.translate(vec3(0, .5, 0));
      ModelTrans.rotate(-90, vec3(0.0, 0.0, 1));
      ModelTrans.scale(2, 2, 1);
      SpecialWall[13].ChangeMatrix(ModelTrans.modelViewMatrix);
      SetModel();
      SetMaterial(RED);
      drawMesh(0);
   ModelTrans.popMatrix();
		
	//Disable the shader
	glUseProgram(0);	
	glutSwapBuffers();

}

/* Reshape - note no scaling as perspective viewing*/
void ReshapeGL (int width, int height)								
{
	g_width = (float)width;
	g_height = (float)height;
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));				

}

/*function to set up looking vectors*/
void initializeVectors() {
	vecW = glm::normalize(-gaze);
	vecU = glm::normalize(glm::cross(glm::vec3(0, 1, 0), vecW));
	vecV = glm::cross(vecW, vecU);
}

/*function to initialize the walls and ground*/ 
void initializeWalls() {
	int i, j;
	
	/*set up static walls*/
	playerBox = StaticBB(eye, 1.f, 1.f, 1.f);
   walls[0] = StaticBB(vec3(0, .5, 2), 1, 1, 1);
   walls[1] = StaticBB(vec3(.1 + groundLen/2, 7.5, 0), 1, 15, groundLen + 3);
   walls[2] = StaticBB(vec3(-1 + -groundLen/2, 7.5, 0), 1, 15, groundLen + 3);
   walls[3] = StaticBB(vec3(0, 7.5, .1 + groundLen/2), groundLen + 3, 15, 1);
   walls[4] = StaticBB(vec3(0, 7.5, -1 + -groundLen/2), groundLen + 3, 15, 1);
   walls[5] = StaticBB(vec3(5, .7, 0), 1, 1, 1);
   walls[6] = StaticBB(vec3(-5, .7, 0), 1, 1, 1);
   
   /*set up special walls*/
   for (i = 0; i < specWallCount; ++i) {
   	SpecialWall[i] = SpecialBB();
   }
   
   /*set up ground*/
   groundArray = (float **)calloc(groundLen, sizeof(float *));
	
	for (i = 0; i < groundLen; ++i) 
		groundArray[i] = (float *)calloc(groundLen, sizeof(float));
	
	for(i = 0; i < groundLen; ++i) {
		for(j = 0; j < groundLen; ++j) {
			groundArray[i][j] = abs(i - groundLen/2)/10.0 + abs(j - groundLen/2)/10.0;
		}
	}
	
	/*start at eye on the ground*/
	eye.y = groundArray[groundLen/2][groundLen/2] + LEGS;
}

/*function to update the players location*/
void RefreshPerson(int value) {
	int tempX = eye.x, tempZ = eye.z, i;
	float x = eye.x, y = eye.y, z = eye.z;
	float floor = groundArray[tempX + groundLen/2][tempZ + groundLen/2];
	vec3 colCheck;
		
	/*add in gravity*/
	velY += gravity*1.5;
	velX = 0;
	velZ = 0;
	
	/*check movement flags triggered from keyboard*/
	if (moveFlags[0]) {
		velX += speed*-vecW.x;
		velZ += speed*-vecW.z;
	}
	if (moveFlags[1]) {
		velX += speed*-vecU.x;
		velZ += speed*-vecU.z;
	}
	if (moveFlags[2]) {
		velX += speed*vecW.x;
		velZ += speed*vecW.z;
	}
	if (moveFlags[3]) {
		velX += speed*vecU.x;
		velZ += speed*vecU.z;
	}
	if (moveFlags[4] && canJump) {
		canJump = false;
		velY += .5;
	}
		
	/*set up the location the eye will be at*/
	x += velX;
	y += velY;
	z += velZ;
	
	/*Collision with walls*/
	for (i = 0; i < wallCount; ++i) {
		colCheck = walls[i].checkCollisionBB(vec3(x, y, z), playerBox);
		if (colCheck.y == 1) {
			y = eye.y;
			velY = 0;
			canJump = true;
		}
		if (colCheck.x == 1) {
			x = eye.x;
		}
		if (colCheck.z == 1) {
			z = eye.z;
		}
	}
	
	/*set bounds on strength of gravity*/
	if (velY < -.1)
		velY = -.1;
		
	/*add in friction*/
	if (velY > .01)
		velY -= .01;
	else if (velY < -.01)
		velY += .01;
	else {
		velY = 0;
	}
		
	/*make walls at the edge*/
	if (x > groundLen/2.0 - 2)
		eye.x = groundLen/2.0 - 2;
	else if (x < -groundLen/2.0 + 2)
		eye.x = -groundLen/2.0 + 2;
	else
		eye.x = x;
		
	if (z > groundLen/2.0 - 2)
		eye.z = groundLen/2.0 - 2;
	else if (z < -groundLen/2.0 + 2)
		eye.z = -groundLen/2.0 + 2;
	else
		eye.z = z;
	
	/*make sure you don't fall through floor*/
	if (y <= floor + LEGS) {
		eye.y = floor + LEGS;
		velY = 0;
		canJump = true;
	} else {
		eye.y = y;
	}
	
	/*update players bounding box*/
	playerBox.updateLoc(eye);
		
	glutPostRedisplay();
}

/*function to update the position of each ball*/
void RefreshBalls(int value) {
   int i, j;
   vec3 temp, hold;
   float floor, ballRad;
   
   /*goes through each ball in the array*/
   for(i = 0; i < ballCount; ++i) {
   	/*apply gravity*/
      bouncies[i].Accelerate(vec3(0, gravity, 0));
      /*apply friction*/
      bouncies[i].velocity *= .99;
      /*get a test location for where the ball will move*/
      temp = bouncies[i].CheckMove();
      ballRad = bouncies[i].rad;
      /*get where it is on the floor*/
      floor = groundArray[(int)(temp.x) + groundLen/2][(int)(temp.z) + groundLen/2];
      
		/*Ball interacts with floor*/
		if (temp.y - ballRad < floor) {
			temp.y = floor + ballRad;
			bouncies[i].Bounce(vec3(0, 1, 0));
		}
	   
      //ball interaction with other balls   
      for (j = 0; j < ballCount; ++j) {
	      if (j != i && bouncies[j].checkCollisionBall(temp, ballRad)) {
	      	//change the velocity of the colliding balls
	      	bouncies[j].velocity += bouncies[i].velocity*.5f;
	         bouncies[i].velocity = -bouncies[i].velocity*.5f;
	         //change the color of the colliding balls
	         ballCols[i] = 3;
	         ballCols[j] = 3;
	         //move the colliding balls away from each other
	         temp += ballRad/2.f*glm::normalize(temp - bouncies[j].loc);
	         bouncies[j].loc += bouncies[j].rad/2.f*glm::normalize(bouncies[j].loc - temp);
	      }
	      
	   }
	   
	   //interaction with Non-axisaligned walls
	   for (j = 0; j < specWallCount; ++j) {
			if(SpecialWall[j].checkCollisionBall(temp, ballRad)) {
				hold = SpecialWall[j].WhichWall(temp, ballRad);
				/*bounce ball off wall and move it out of wall*/
				bouncies[i].Bounce(glm::normalize(hold));
				temp += hold;
				//change ball color
				if (j > 5)
			   	ballCols[i] = 7;
			   else
					ballCols[i] = 4;
			}
		}
	   
	   //balls interaction with rotating walls
	   for (j = 0; j < wallCount; ++j) {
			if (walls[j].checkCollisionBall(temp, ballRad)) {
				hold = walls[j].WhichWall(temp, ballRad);
				/*bounce ball off wall and move it out of wall*/
			   bouncies[i].Bounce(glm::normalize(hold));
			   temp += hold;
			   //change ball color
			   ballCols[i] = 4;
			}
	   }
	   
	   //balls interact with player
	   if (playerBox.checkCollisionBall(temp, ballRad)) {
			hold = playerBox.WhichWall(temp, ballRad);
			/*bounce ball off of player*/
		   bouncies[i].Bounce(glm::normalize(hold));
		   temp += hold;
		   //change ball color
		   ballCols[i] = 7;
		}
      
      /*move ball to new location*/
		bouncies[i].MoveTo(temp);
	}
}

/*function to rotate all the rotating walls*/
void RotateStuff(int value) {
	rotAng += .3;
	glutTimerFunc(20, RotateStuff, 0);
}

/*function to refresh all the balls and the person at the same time*/
void RefreshWorld(int value) {
	glutTimerFunc(TIMER_MS, RefreshWorld, 0);
	glutTimerFunc(TIMER_MS, RefreshBalls, 0);
  	glutTimerFunc(TIMER_MS, RefreshPerson, 0);
}

int main( int argc, char *argv[] )
{
   glutInit( &argc, argv );
   glutInitWindowPosition( 20, 20 );
   glutInitWindowSize( 600, 600 );
   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   glutCreateWindow("Dodge balls!");
   glutReshapeFunc( ReshapeGL );
   glutDisplayFunc( Draw );
   glutKeyboardFunc( keyboard );
   glutKeyboardUpFunc( letgo );
   glutMouseFunc( Mouse );
   glutMotionFunc( Motion );
   Initialize();
   initializeVectors();
   initializeWalls();
	
	//test the openGL version
	getGLversion();
	//install the shader
	if (!InstallShader(textFileRead((char *)"Lab4_vert.glsl"), textFileRead((char *)"Lab4_frag.glsl"))) {
		printf("Error installing shader!\n");
		return 0;
	}
		
	InitGeom();
	glutTimerFunc(TIMER_MS, RefreshWorld, 0);
	glutTimerFunc(20, RotateStuff, 0);
  	glutMainLoop();
   return 0;
}
