struct Material {
  vec3 aColor;
  vec3 dColor;
  vec3 sColor;
  float shine;
};

const int lightCt = 1;

uniform mat4 uProjMatrix;
uniform mat4 uViewMatrix;
uniform mat4 uModelMatrix;

uniform vec3 uLightPosArray[lightCt];
uniform vec3 uLightColArray[lightCt];

uniform int lightCount;

attribute vec3 aNormal;
attribute vec3 aPosition;

varying vec4 tnormal;
varying vec4 light[lightCt];
varying vec4 v;
varying vec4 Refl[lightCt];
varying float dist[lightCt];


void main() {
  	vec4 vPosition;
	vec3 Spec, Diffuse;
	float dotNormLigh, dotVRefl;
	int i = 0;

	/* First model transforms */
	vPosition = uModelMatrix*vec4(aPosition.x, aPosition.y, aPosition.z, 1);
	 
	for (i = 0; i < lightCt; ++i) {
		/*set up lighting before camera and after translations*/
		light[i] = (vec4(uLightPosArray[i], 1) - vPosition); //the vector of light position - v position
		light[i] = normalize(light[i]);
	}

	/*set up the normal to work with transformations, note that inverse and transpose have minimum version requirements of 150 or later.*/
	tnormal = uModelMatrix*vec4(aNormal, 0.0);
	tnormal = normalize(tnormal);  
	
	for (i = 0; i < lightCt; ++i) {  
		/*calculate reflection*/
		Refl[i] = -light[i] + 2.0*(clamp(dot(tnormal, light[i]), 0.0, 1.0))*tnormal; 
		/*calculate the distance*/
		dist[i] = distance(vec3(vPosition), uLightPosArray[i]);
		dist[i] = (1.0/(dist[i]));
	}
	  
	vPosition = uViewMatrix*vPosition;
	
	/*calculate v*/
	v = normalize(vec4(0,0,0,1.0) - vPosition);

	gl_Position = uProjMatrix*vPosition;
}
